@@ -22,12 +22,12 @@ Read the user manual for a description of the interface to this scoring engine
 """
 
 import re # the new regex is about 7% faster
-from hashlib import md5 # pylint: disable=E0611
+from hashlib import md5
 
-from PyQt4.QtCore import QString, QVariant
+from PyQt4.QtCore import QVariant
 
 from util import m18n, m18nc, m18nE, english, logException
-from query import Query
+from query import Query, Transaction
 
 import rulecode
 
@@ -38,6 +38,7 @@ class Score(object):
     should want to set more than one unit, split it into two rules.
     For the first use case only we have the attributes value and unit"""
 
+    __hash__ = None
 
     def __init__(self, points=0, doubles=0, limits=0, ruleset=None):
         self.points = 0 # define the types for those values
@@ -83,6 +84,9 @@ class Score(object):
             parts.append('limits=%f' % self.limits)
         return ' '.join(parts)
 
+    def __repr__(self):
+        return 'Score(%s)' % str(self)
+
     def contentStr(self):
         """make score readable for humans, i18n"""
         parts = []
@@ -157,40 +161,46 @@ class RuleList(list):
         self.name = name
         self.description = description
 
-    def pop(self, name):
+    def pop(self, key):
         """find rule, return it, delete it from this list"""
-        result = self.__getitem__(name)
-        self.__delitem__(name)
+        result = self.__getitem__(key)
+        self.__delitem__(key)
         return result
 
-    def __getitem__(self, name):
-        """find rule by name"""
-        if isinstance(name, int):
-            return list.__getitem__(self, name)
+    def __contains__(self, key):
+        """do we know this rule?"""
+        if isinstance(key, Rule):
+            key = key.key()
+        return any(x.key() == key for x in self)
+
+    def __getitem__(self, key):
+        """find rule by key"""
+        if isinstance(key, int):
+            return list.__getitem__(self, key)
         for rule in self:
-            if rule.name == name:
+            if rule.key() == key:
                 return rule
         raise KeyError
 
-    def __setitem__(self, name, rule):
-        """set rule by name"""
+    def __setitem__(self, key, rule):
+        """set rule by key"""
         assert isinstance(rule, Rule)
-        if isinstance(name, int):
-            list.__setitem__(self, name, rule)
+        if isinstance(key, int):
+            list.__setitem__(self, key, rule)
             return
         for idx, oldRule in enumerate(self):
-            if oldRule.name == name:
+            if oldRule.key() == key:
                 list.__setitem__(self, idx, rule)
                 return
         list.append(self, rule)
 
-    def __delitem__(self, name):
+    def __delitem__(self, key):
         """delete this rule"""
-        if isinstance(name, int):
-            list.__delitem__(self, name)
+        if isinstance(key, int):
+            list.__delitem__(self, key)
             return
         for idx, rule in enumerate(self):
-            if rule.name == name:
+            if rule.key() == key:
                 list.__delitem__(self, idx)
                 return
         raise KeyError
@@ -201,7 +211,10 @@ class RuleList(list):
 
     def add(self, rule):
         """use add instead of append"""
-        self[rule.name] = rule
+        if rule.key() in self:
+            logException('%s is already defined as %s, not accepting new rule %s/%s'% (
+                rule.key(), self[rule.key()].definition, rule.name, rule.definition))
+        self[rule.key()] = rule
 
 class Ruleset(object):
     """holds a full set of rules: splitRules,meldRules,handRules,winnerRules.
@@ -218,11 +231,14 @@ class Ruleset(object):
         all of its rules. This excludes the splitting rules, IOW exactly the rules saved in the table
         rule will be used for computation.
 
-        used rulesets and rules are stored in separate tables - this makes handling them easier.
-        In table usedruleset the name is not unique.
+        Rulesets which are templates for new games have negative ids.
+        Rulesets attached to a game have positive ids.
+
+        The name is not unique.
     """
-    # pylint: disable=R0902
-    # pylint we need more than 10 instance attributes
+    # pylint: disable=too-many-instance-attributes
+
+    __hash__ = None
 
     cache = dict()
     hits = 0
@@ -234,21 +250,30 @@ class Ruleset(object):
         Ruleset.cache.clear()
 
     @staticmethod
-    def cached(name, used=False):
+    def cached(name):
         """If a Ruleset instance is never changed, we can use a cache"""
+        if isinstance(name, list):
+            # we got the rules over the wire
+            _, wiredHash, _, _ = name[0]
+        else:
+            wiredHash = None
+        for predefined in PredefinedRuleset.rulesets():
+            if predefined.hash in (name, wiredHash):
+                return predefined
         cache = Ruleset.cache
-        cacheKey = str(name) + str(used)
-        if cacheKey in cache:
-            return cache[cacheKey]
-        result = Ruleset(name, used)
-        cache[cacheKey] = result
+        if not isinstance(name, list) and name in cache:
+            return cache[name]
+        result = Ruleset(name)
+        cache[result.rulesetId] = result
+        cache[result.hash] = result
         return result
 
-
-    def __init__(self, name, used=False):
+    def __init__(self, name):
+        """name may be:
+            - an integer: ruleset.id from the sql table
+            - a list: the full ruleset specification (probably sent from the server)
+            - a string: The hash value of a ruleset"""
         self.name = name
-        self.__used = used
-        self.orgUsed = used
         self.rulesetId = 0
         self.__hash = None
         self.allRules = []
@@ -281,30 +306,26 @@ into a situation where you have to pay a penalty"""))
         # the order of ruleLists is the order in which the lists appear in the ruleset editor
         # if you ever want to remove an entry from ruleLists: make sure its listId is not reused or you get
         # in trouble when updating
-        self.initRuleset()
+        self._initRuleset()
 
-    @apply
-    def dirty(): # pylint: disable=E0202
+    @property
+    def dirty(self):
         """have we been modified since load or last save?"""
-        def fget(self):
-            # pylint: disable=W0212
-            return self.__dirty
-        def fset(self, dirty):
-            # pylint: disable=W0212
-            self.__dirty = dirty
-            if dirty:
-                self.__computeHash()
-        return property(**locals())
+        return self.__dirty
 
-    @apply
-    def hash():
+    @dirty.setter
+    def dirty(self, dirty):
+        """have we been modified since load or last save?"""
+        self.__dirty = dirty
+        if dirty:
+            self.__computeHash()
+
+    @property
+    def hash(self):
         """a md5sum computed from the rules but not name and description"""
-        def fget(self):
-            # pylint: disable=W0212
-            if not self.__hash:
-                self.__computeHash()
-            return self.__hash
-        return property(**locals())
+        if not self.__hash:
+            self.__computeHash()
+        return self.__hash
 
 
     def __eq__(self, other):
@@ -319,28 +340,37 @@ into a situation where you have to pay a penalty"""))
         We only use this for scoring games."""
         return self.minMJPoints + min(x.score.total() for x in self.mjRules)
 
-    def initRuleset(self):
+    @staticmethod
+    def hashIsKnown(value):
+        """returns False or True"""
+        result = any(x.hash == value for x in PredefinedRuleset.rulesets())
+        if not result:
+            query = Query("select id from ruleset where hash=?", list([value]))
+            result = bool(query.records)
+        return result
+
+    def _initRuleset(self):
         """load ruleset headers but not the rules"""
         if isinstance(self.name, int):
-            query = Query("select id,name,description from %s where id=%d" % \
-                          (self.__rulesetTable(), self.name))
+            query = Query("select id,hash,name,description from ruleset where id=%d" % self.name)
         elif isinstance(self.name, list):
             # we got the rules over the wire
             self.rawRules = self.name[1:]
-            (self.rulesetId, self.name, self.description) = self.name[0]
+            (self.rulesetId, self.__hash, self.name, self.description) = self.name[0]
+            self.load() # load raw rules at once, rules from db only when needed
             return
         else:
-            query = Query("select id,name,description from %s where name=?" % \
-                          self.__rulesetTable(), list([self.name]))
+            query = Query("select id,hash,name,description from ruleset where hash=?",
+                          list([self.name]))
         if len(query.records):
-            (self.rulesetId, self.name, self.description) = query.records[0]
+            (self.rulesetId, self.__hash, self.name, self.description) = query.records[0]
         else:
-            raise Exception(m18n('ruleset "%1" not found', self.name))
+            raise Exception('ruleset %s not found' % self.name)
 
     def load(self):
-        """load the ruleset from the database and compute the hash"""
+        """load the ruleset from the database and compute the hash. Return self."""
         if self.__loaded:
-            return
+            return self
         self.__loaded = True
         # we might have introduced new mandatory rules which do
         # not exist in the rulesets saved with the games, so preload
@@ -358,43 +388,35 @@ into a situation where you have to pay a penalty"""))
         for par in self.parameterRules:
             self.__dict__[par.parName] = par.parameter
         for ruleList in self.ruleLists:
+            assert len(ruleList) == len(set(x.key() for x in ruleList)), '%s has non-unique key' % ruleList.name
             for rule in ruleList:
                 rule.score.ruleset = self
                 self.allRules.append(rule)
         self.doublingMeldRules = list(x for x in self.meldRules if x.score.doubles)
         self.doublingHandRules = list(x for x in self.handRules if x.score.doubles)
+        return self
 
-    def loadQuery(self):
+    def __loadQuery(self):
         """returns a Query object with loaded ruleset"""
-        return Query("select ruleset, name, list, position, definition, points, doubles, limits, parameter from %s "
-                "where ruleset=%d order by list,position" % \
-                      (self.__ruleTable(), self.rulesetId))
-
-    @staticmethod
-    def fromList(source):
-        """returns a Ruleset as defined by the list source"""
-        result = Ruleset(source)
-        for predefined in PredefinedRuleset.rulesets():
-            if result == predefined:
-                return predefined
-        return result
+        return Query(
+            "select ruleset, name, list, position, definition, points, doubles, limits, parameter from rule "
+                "where ruleset=%d order by list,position" % self.rulesetId)
 
     def toList(self):
         """returns entire ruleset encoded in a string"""
         self.load()
-        result = [[self.rulesetId, self.name, self.description]]
-        result.extend(self.ruleRecords())
+        result = [[self.rulesetId, self.hash, self.name, self.description]]
+        result.extend(self.ruleRecord(x) for x in self.allRules)
         return result
 
     def loadRules(self):
         """load rules from database or from self.rawRules (got over the net)"""
-        for record in self.rawRules or self.loadQuery().records:
-            self.loadRule(record)
+        for record in self.rawRules or self.__loadQuery().records:
+            self.__loadRule(record)
 
-    def loadRule(self, record):
+    def __loadRule(self, record):
         """loads a rule into the correct ruleList"""
-        (rulesetIdx, name, listNr, position, definition, points, doubles, limits, # pylint: disable=W0612
-            parameter) = record
+        (_, name, listNr, _, definition, points, doubles, limits, parameter) = record
         for ruleList in self.ruleLists:
             if ruleList.listId == listNr:
                 if ruleList is self.parameterRules:
@@ -410,23 +432,15 @@ into a situation where you have to pay a penalty"""))
                 ruleList.add(rule)
                 break
 
-    def findRule(self, name):
-        """return the rule named 'name'. Also finds it if the rule definition starts with name"""
-        for ruleList in self.ruleLists:
-            for rule in ruleList:
-                if rule.name == name or rule.definition.startswith(name):
-                    return rule
-        raise Exception('no rule found:' + name)
-
-    def findAction(self, action):
-        """return first rule with action"""
+    def findUniqueOption(self, action):
+        """return first rule with option"""
         rulesWithAction = list(x for x in self.allRules if action in x.options)
         assert len(rulesWithAction) < 2, '%s has too many matching rules for %s' % (str(self), action)
         if rulesWithAction:
             return rulesWithAction[0]
 
     def filterFunctions(self, attrName):
-        """returns all my rules having a function with an attribute named attrName"""
+        """returns all my Function classes having attribute attrName"""
         if attrName not in self.__filteredLists:
             functions = (x.function for x in self.allRules if x.function)
             self.__filteredLists[attrName] = list(x for x in functions if hasattr(x, attrName))
@@ -446,57 +460,55 @@ into a situation where you have to pay a penalty"""))
         self.splitRules.append(Splitter('pair', r'([DWSBCdwsbc][1-9eswnbrg])(\1)', 2))
         self.splitRules.append(Splitter('single', r'(..)', 1))
 
-    def newId(self, used=None):
+    @staticmethod
+    def newId(minus=False):
         """returns an unused ruleset id. This is not multi user safe."""
-        if used is not None:
-            self.__used = used
-        records = Query("select max(id)+1 from %s" % self.__rulesetTable()).records
+        func = 'min(id)-1' if minus else 'max(id)+1'
+        result = -1 if minus else 1
+        records = Query("select %s from ruleset" % func).records
         if records and records[0] and records[0][0]:
             try:
-                return int(records[0][0])
+                result = int(records[0][0])
             except ValueError:
-                return 1
-        return 1
+                pass
+        return result
 
     @staticmethod
-    def nameIsDuplicate(name):
-        """show message and raise Exception if ruleset name is already in use"""
-        return bool(Query('select id from ruleset where name=?', list([name])).records)
+    def nameExists(name):
+        """return True if ruleset name is already in use"""
+        result = any(x.name == name for x in PredefinedRuleset.rulesets())
+        if not result:
+            result = bool(Query('select id from ruleset where id<0 and name=?', list([name])).records)
+        return result
 
-    def _newKey(self):
-        """returns a new key and a new name for a copy of self"""
-        newId = self.newId()
-        for copyNr in range(1, 100):
-            copyStr = ' ' + str(copyNr) if copyNr > 1 else ''
-            newName = m18nc('Ruleset._newKey:%1 is empty or space plus number',
-                'Copy%1 of %2', copyStr, m18n(self.name))
-            if not self.nameIsDuplicate(newName):
-                return newId, newName
-        logException('You already have the maximum number of copies, please rename some')
+    def _newKey(self, minus=False):
+        """generate a new id and a new name if the name already exists"""
+        newId = self.newId(minus=minus)
+        newName = self.name
+        if minus:
+            copyNr = 1
+            while self.nameExists(newName):
+                copyStr = ' ' + str(copyNr) if copyNr > 1 else ''
+                newName = m18nc('Ruleset._newKey:%1 is empty or space plus number',
+                    'Copy%1 of %2', copyStr, m18n(self.name))
+                copyNr += 1
+        return newId, newName
 
     def clone(self):
         """returns a clone of self, unloaded"""
         return Ruleset(self.rulesetId)
 
     def __str__(self):
-        return 'type=%s, id=%d,rulesetId=%d,name=%s,used=%d' % (
-                type(self), id(self), self.rulesetId, self.name, self.__used)
+        return 'type=%s, id=%d,rulesetId=%d,name=%s' % (
+                type(self), id(self), self.rulesetId, self.name)
 
-    def copy(self):
-        """make a copy of self and return the new ruleset id. Returns a new ruleset Id or None"""
-        newRuleset = self.clone()
-        newRuleset.load()
-        if newRuleset.saveCopy():
-            if isinstance(newRuleset, PredefinedRuleset):
-                newRuleset = Ruleset(newRuleset.rulesetId)
-            return newRuleset
-
-    def saveCopy(self):
-        """give this ruleset a new id and a new name and save it"""
-        assert not self.__used
-        self.rulesetId, self.name = self._newKey()
-        self.dirty = True # does not yet exist
-        return self.save()
+    def copy(self, minus=False):
+        """make a copy of self and return the new ruleset id. Returns a new ruleset or None"""
+        newRuleset = self.clone().load()
+        newRuleset.save(copy=True, minus=minus)
+        if isinstance(newRuleset, PredefinedRuleset):
+            newRuleset = Ruleset(newRuleset.rulesetId)
+        return newRuleset
 
     def __ruleList(self, rule):
         """return the list containg rule. We could make the list
@@ -507,28 +519,21 @@ into a situation where you have to pay a penalty"""))
                 return ruleList
         assert False
 
-    def __rulesetTable(self):
-        """the table name for the ruleset"""
-        return 'usedruleset' if self.__used else 'ruleset'
-
-    def __ruleTable(self):
-        """the table name for the rule"""
-        return 'usedrule' if self.__used else 'rule'
-
     def rename(self, newName):
         """renames the ruleset. returns True if done, False if not"""
-        if self.nameIsDuplicate(newName):
-            return False
-        query = Query("update ruleset set name=? where name =?",
-            list([newName, self.name]))
-        if query.success:
-            self.name = newName
-        return query.success
+        with Transaction():
+            if self.nameExists(newName):
+                return False
+            query = Query("update ruleset set name=? where id<0 and name =?",
+                list([newName, self.name]))
+            if query.success:
+                self.name = newName
+            return query.success
 
     def remove(self):
         """remove this ruleset from the database."""
-        Query(["DELETE FROM %s WHERE ruleset=%d" % (self.__ruleTable(), self.rulesetId),
-                   "DELETE FROM %s WHERE id=%d" % (self.__rulesetTable(), self.rulesetId)])
+        Query(["DELETE FROM rule WHERE ruleset=%d" % self.rulesetId,
+                   "DELETE FROM ruleset WHERE id=%d" % self.rulesetId])
 
     @staticmethod
     def ruleKey(rule):
@@ -544,53 +549,58 @@ into a situation where you have to pay a penalty"""))
             result.update(rule.hashStr())
         self.__hash = result.hexdigest()
 
-    def ruleRecords(self):
-        """returns a list of all rules, prepared for use by sql"""
-        parList = []
+    def ruleRecord(self, rule):
+        """returns the rule as tuple, prepared for use by sql"""
+        score = rule.score
+        definition = rule.definition
+        if rule.parType:
+            parTypeName = rule.parType.__name__
+            if parTypeName == 'unicode':
+                parTypeName = 'str'
+            definition = parTypeName + definition
+        ruleList = None
         for ruleList in self.ruleLists:
-            for ruleIdx, rule in enumerate(ruleList):
-                score = rule.score
-                definition = rule.definition
-                if rule.parType:
-                    parTypeName = rule.parType.__name__
-                    if parTypeName == 'unicode':
-                        parTypeName = 'str'
-                    definition = parTypeName + definition
-                parList.append(list([self.rulesetId, english(rule.name), ruleList.listId, ruleIdx,
-                    definition, score.points, score.doubles, score.limits, str(rule.parameter)]))
-        return parList
+            if rule in ruleList:
+                ruleIdx = ruleList.index(rule)
+                break
+        assert rule in ruleList
+        return (self.rulesetId, english(rule.name), ruleList.listId, ruleIdx,
+            definition, score.points, score.doubles, score.limits, str(rule.parameter))
 
-    def save(self):
-        """save the ruleset to the database"""
-        if not self.dirty and self.__used == self.orgUsed:
-            # same content in same table
-            return True
-        Query.dbhandle.transaction()
-        self.remove()
-        if not Query('INSERT INTO %s(id,name,hash,description) VALUES(?,?,?,?)' % self.__rulesetTable(),
-            list([self.rulesetId, english(self.name).decode('utf-8'), self.hash, self.description])).success:
-            Query.dbhandle.rollback()
-            return False
-        result = Query('INSERT INTO %s(ruleset, name, list, position, definition, '
+    def updateRule(self, rule):
+        """update rule in database"""
+        self.__hash = None  # invalidate, will be recomputed when needed
+        with Transaction():
+            Query("DELETE FROM rule WHERE ruleset=? and name=?", list([self.rulesetId, english(rule.name)]))
+            self.saveRule(rule)
+            Query("UPDATE ruleset SET hash=? WHERE id=?", list([self.hash, self.rulesetId]))
+
+    def saveRule(self, rule):
+        """save only rule in database"""
+        Query('INSERT INTO rule(ruleset, name, list, position, definition, '
                 'points, doubles, limits, parameter)'
-                ' VALUES(?,?,?,?,?,?,?,?,?)' % self.__ruleTable(),
-                self.ruleRecords()).success
-        if result:
-            Query.dbhandle.commit()
-            self.dirty = False
-        else:
-            Query.dbhandle.rollback()
-        return result
+                ' VALUES(?,?,?,?,?,?,?,?,?)',
+                self.ruleRecord(rule))
 
-    @staticmethod
-    def availableRulesetNames():
-        """returns all ruleset names defined in the database"""
-        return list(x[0] for x in Query("SELECT name FROM ruleset").records)
+    def save(self, copy=False, minus=False):
+        """save the ruleset to the database.
+        copy=True gives it a new id. If the name already exists in the database, also give it a new name"""
+        with Transaction():
+            if copy:
+                self.rulesetId, self.name = self._newKey(minus)
+            Query('INSERT INTO ruleset(id,name,hash,description) VALUES(?,?,?,?)',
+                list([self.rulesetId, english(self.name), self.hash, self.description]))
+        # do not put this into the transaction, keep it as short as possible. sqlite3/Qt
+        # has problems if two processes are trying to do the same here (kajonggtest)
+        for rule in self.allRules:
+            self.saveRule(rule)
 
     @staticmethod
     def availableRulesets():
         """returns all rulesets defined in the database plus all predefined rulesets"""
-        return [Ruleset(x) for x in Ruleset.availableRulesetNames()] + PredefinedRuleset.rulesets()
+        # TODO: this is done twice on the client with --demo, why?
+        templateIds = (x[0] for x in Query("SELECT id FROM ruleset WHERE id<0").records)
+        return [Ruleset(x) for x in templateIds] + PredefinedRuleset.rulesets()
 
     @staticmethod
     def selectableRulesets(server=None):
@@ -604,7 +614,7 @@ into a situation where you have to pay a penalty"""))
         if server is None: # scoring game
             # the exists clause is only needed for inconsistent data bases
             qData = Query("select ruleset from game where seed is null "
-                " and exists(select id from usedruleset where game.ruleset=usedruleset.id)"
+                " and exists(select id from ruleset where game.ruleset=ruleset.id)"
                 "order by starttime desc limit 1").records
         else:
             qData = Query('select lastruleset from server where lastruleset is not null and url=?',
@@ -614,7 +624,8 @@ into a situation where you have to pay a penalty"""))
                 qData = Query('select lastruleset from server where lastruleset is not null '
                     'order by lasttime desc limit 1').records
         if qData:
-            qData = Query("select name from usedruleset where id=%d" % qData[0][0]).records
+            lastUsedId = qData[0][0]
+            qData = Query("select name from ruleset where id=%d" % lastUsedId).records
             if qData:
                 lastUsed = qData[0][0]
                 for idx, ruleset in enumerate(result):
@@ -645,15 +656,15 @@ class Rule(object):
     The rule applies if at least one of the variants matches the hand.
     For parameter rules, only use name, definition,parameter. definition must start with int or str
     which is there for loading&saving, but internally is stripped off."""
-    # pylint: disable=R0913,R0902
-    # pylint we need more than 10 instance attributes
+    # pylint: disable=too-many-arguments,too-many-instance-attributes
 
     def __init__(self, name, definition='', points = 0, doubles = 0, limits = 0, parameter = None,
-            description=None, debug=False):
+            description=None, explainTemplate=None, debug=False):
         self.options = {}
         self.function = None
         self.hasSelectable = False
         self.name = name
+        self.explainTemplate = explainTemplate
         self.description = description
         self.score = Score(points, doubles, limits)
         self._definition = None
@@ -674,61 +685,73 @@ class Rule(object):
                 break
         self.definition = definition
 
-    @apply
-    def definition(): # pylint: disable=E0202
+    def key(self):
+        """the key is used for finding a rule in a RuleList. Since we do not
+        want to manually define a unique key for every rule, this is a bit
+        complicated:
+        - if the definition starts with F and contains no ||, key is rule.definition
+          without the leading F
+        - if the definition defines a parameter, the parType at the beginning is stripped
+          and the first part before || is used
+        - otherwise key is rule.name"""
+        if not self._definition:
+            return self.name
+        if self._definition[0] == 'F' and '||' not in self._definition:
+            return self._definition[1:]
+        elif self.parType:
+            return self._definition.split('||')[0]
+        return self.name
+
+    @property
+    def definition(self):
         """the rule definition. See user manual about ruleset."""
-        # pylint: disable=R0912
-        def fget(self):
-            # pylint: disable=W0212
-            if isinstance(self._definition, list):
-                return '||'.join(self._definition)
-            return self._definition
-        def fset(self, definition):
-            """setter for definition"""
-            # pylint: disable=W0212
-            assert not isinstance(definition, QString)
-            prevDefinition = self.definition
-            self._definition = definition
-            if not definition:
-                return # may happen with special programmed rules
-            variants = definition.split('||')
-            if self.parType:
-                self.parName = variants[0]
-                variants = variants[1:]
-            self.options = {}
-            self.function = None
-            self.hasSelectable = False
-            for idx, variant in enumerate(variants):
-                if isinstance(variant, (str, unicode)):
-                    variant = str(variant)
-                    if variant[0] == 'F':
-                        assert idx == 0
-                        self.function = rulecode.Function.functions[variant[1:]]()
-                        # when executing code for this rule, we do not want
-                        # to call those things indirectly
-                        if hasattr(self.function, 'appliesToHand'):
-                            self.appliesToHand = self.function.appliesToHand
-                        if hasattr(self.function, 'appliesToMeld'):
-                            self.appliesToMeld = self.function.appliesToMeld
-                        if hasattr(self.function, 'selectable'):
-                            self.hasSelectable = True
-                            self.selectable = self.function.selectable
-                        if hasattr(self.function, 'winningTileCandidates'):
-                            self.winningTileCandidates = self.function.winningTileCandidates
-                    elif variant[0] == 'O':
-                        for action in variant[1:].split():
-                            aParts = action.split('=')
-                            if len(aParts) == 1:
-                                aParts.append('None')
-                            self.options[aParts[0]] = aParts[1]
-                    elif variant == 'XEAST9X':
-                        pass
-                    else:
-                        pass
-            if self.function:
-                self.function.options = self.options
-            self.validateDefinition(prevDefinition)
-        return property(**locals())
+        if isinstance(self._definition, list):
+            return '||'.join(self._definition)
+        return self._definition
+
+    @definition.setter
+    def definition(self, definition):
+        """setter for definition"""
+        #pylint: disable=too-many-branches
+        prevDefinition = self.definition
+        self._definition = definition
+        if not definition:
+            return # may happen with special programmed rules
+        variants = definition.split('||')
+        if self.parType:
+            self.parName = variants[0]
+            variants = variants[1:]
+        self.options = {}
+        self.function = None
+        self.hasSelectable = False
+        for idx, variant in enumerate(variants):
+            if isinstance(variant, (str, unicode)):
+                variant = str(variant)
+                if variant[0] == 'F':
+                    assert idx == 0
+                    self.function = rulecode.Function.functions[variant[1:]]()
+                    # when executing code for this rule, we do not want
+                    # to call those things indirectly
+                    if hasattr(self.function, 'appliesToHand'):
+                        self.appliesToHand = self.function.appliesToHand
+                    if hasattr(self.function, 'appliesToMeld'):
+                        self.appliesToMeld = self.function.appliesToMeld
+                    if hasattr(self.function, 'selectable'):
+                        self.hasSelectable = True
+                        self.selectable = self.function.selectable
+                    if hasattr(self.function, 'winningTileCandidates'):
+                        self.winningTileCandidates = self.function.winningTileCandidates
+                elif variant[0] == 'O':
+                    for action in variant[1:].split():
+                        aParts = action.split('=')
+                        if len(aParts) == 1:
+                            aParts.append('None')
+                        self.options[aParts[0]] = aParts[1]
+                else:
+                    pass
+        if self.function:
+            self.function.options = self.options
+        self.validateDefinition(prevDefinition)
 
     def validateDefinition(self, prevDefinition):
         """check for validity. If wrong, restore prevDefinition."""
@@ -747,25 +770,32 @@ class Rule(object):
                 return m18nc('wrong value for rule', '%1: %2 is too small, minimal value is %3',
                     m18n(self.name), self.parameter, minValue)
 
-    def appliesToHand(self, dummyHand): # pylint: disable=R0201
+    def appliesToHand(self, dummyHand): # pylint: disable=no-self-use, E0202
         """does the rule apply to this hand?"""
         return False
 
-    def selectable(self, dummyHand): # pylint: disable=R0201
+    def selectable(self, dummyHand): # pylint: disable=no-self-use, E0202
         """does the rule apply to this hand?"""
         return False
 
-    def appliesToMeld(self, dummyHand, dummyMeld): # pylint: disable=R0201
+    def appliesToMeld(self, dummyHand, dummyMeld): # pylint: disable=no-self-use, E0202
         """does the rule apply to this meld?"""
         return False
 
-    def winningTileCandidates(self, dummyHand): # pylint: disable=R0201
+    def winningTileCandidates(self, dummyHand): # pylint: disable=no-self-use, E0202
         """those might be candidates for a calling hand"""
         return set()
 
-    def explain(self):
+    def explain(self, meld):
         """use this rule for scoring"""
-        return m18n(self.name) + ': ' + self.score.contentStr()
+        return '%s: %s' % ( m18n(
+            self.explainTemplate if self.explainTemplate else self.name).format(
+                group=meld[0].groupName() if meld else '',
+                value=meld[0].valueName() if meld else '',
+                meldType=meld.typeName() if meld else '',
+                meldName=meld.name() if meld else '',
+                tileName=meld[0].name() if meld else ''
+            ).replace('&', '').replace('  ', ' ').strip(), self.score.contentStr())
 
     def hashStr(self):
         """all that is needed to hash this rule. Try not to change this to keep
@@ -827,6 +857,7 @@ class PredefinedRuleset(Ruleset):
     """special code for loading rules from program code instead of from the database"""
 
     classes = set()  # only those will be playable
+    preRulesets = None
 
     def __init__(self, name=None):
         Ruleset.__init__(self, name or 'general predefined ruleset')
@@ -834,7 +865,10 @@ class PredefinedRuleset(Ruleset):
     @staticmethod
     def rulesets():
         """a list of instances for all predefined rulesets"""
-        return list(x() for x in PredefinedRuleset.classes)
+        if PredefinedRuleset.preRulesets is None:
+            PredefinedRuleset.preRulesets = list(x()
+                for x in sorted(PredefinedRuleset.classes, key=lambda x:x.__name__))
+        return PredefinedRuleset.preRulesets
 
     def rules(self):
         """here the predefined rulesets can define their rules"""
