@@ -38,15 +38,16 @@ from modeltest import ModelTest
 from rulesetselector import RuleTreeView
 from board import WindLabel, WINDPIXMAPS
 from util import m18n, m18nc
-from common import WINDS, InternalParameters, Debug
+from common import WINDS, Internal, Debug
 from statesaver import StateSaver
 from query import Query
 from guiutil import ListComboBox
 from tree import TreeItem, RootItem, TreeModel
+from tile import Tile
 
 class ScoreTreeItem(TreeItem):
     """generic class for items in our score tree"""
-    # pylint: disable=W0223
+    # pylint: disable=abstract-method
     # we know content() is abstract, this class is too
 
     def columnCount(self):
@@ -88,7 +89,12 @@ class ScorePlayerItem(ScoreTreeItem):
         if column == 0:
             return m18n(self.rawContent[0])
         else:
-            return self.hands()[column-1]
+            try:
+                return self.hands()[column-1]
+            except IndexError:
+                # we have a penalty but no hand yet. Should
+                # not happen in practical use
+                return None
 
     def hands(self):
         """a small helper"""
@@ -143,7 +149,7 @@ class ScoreItemDelegate(QStyledItemDelegate):
                     # if we want to use a pen width > 1, we can no longer directly drawPolyline
                     # separately per cell beause the lines spread vertically over two rows: We would
                     # have to draw the lines into one big pixmap and copy from the into the cells
-                    painter.drawPolyline(*chart) # pylint: disable=W0142
+                    painter.drawPolyline(*chart) # pylint: disable=star-args
                     painter.restore()
             return
         return QStyledItemDelegate.paint(self, painter, option, index)
@@ -169,10 +175,9 @@ class ScoreModel(TreeModel):
         xValues = list(x * stepX for x in range(self.steps + 1))
         return list(QPointF(x, y) for x, y in zip(xValues, yValues))
 
-    def data(self, index, role=None): # pylint: disable=R0201
+    def data(self, index, role=None): # pylint: disable=no-self-use,too-many-branches
         """score table"""
-        # pylint: disable=R0911,R0912
-        # pylint - too many returns and branches
+        # pylint: disable=too-many-return-statements
         if not index.isValid():
             return QVariant()
         column = index.column()
@@ -211,10 +216,10 @@ class ScoreModel(TreeModel):
                     return QVariant(QBrush(ScoreItemDelegate.colors[index.row()]))
         if column > 0 and isinstance(item, ScorePlayerItem) :
             content = item.content(column)
-            # pylint: disable=E1103
+            # pylint: disable=maybe-no-member
             # pylint thinks content is a str
             if role == Qt.BackgroundRole:
-                if content.won:
+                if content and content.won:
                     return QVariant(QColor(165, 255, 165))
             if role == Qt.ToolTipRole:
                 englishHints = content.manualrules.split('||')
@@ -249,10 +254,11 @@ class ScoreModel(TreeModel):
                 'select player,rotated,notrotated,penalty,won,prevailing,wind,points,payments,balance,manualrules'
                 ' from score where game=? order by player,hand',
                 list([game.gameid])).records
-        # pylint: disable=W0142
-        # pylint * magic
+        # pylint: disable=star-args
+        humans = sorted(x for x in game.players if not x.name.startswith('Robot'))
+        robots = sorted(x for x in game.players if x.name.startswith('Robot'))
         data = list(tuple([player.localName, [HandResult(*x[1:]) for x in records \
-                if x[0] == player.nameid]]) for player in game.players)
+                if x[0] == player.nameid]]) for player in humans + robots)
         self.__findMinMaxChartPoints(data)
         parent = QModelIndex()
         groupIndex = self.index(self.rootItem.childCount(), 0, parent)
@@ -280,7 +286,7 @@ class ScoreModel(TreeModel):
 
 class HandResult(object):
     """holds the results of a hand for the scoring table"""
-    # pylint: disable=R0913
+    # pylint: disable=too-many-arguments
     # we have too many arguments
     def __init__(self, rotated, notRotated, penalty, won, prevailing, wind, points, payments, balance, manualrules):
         self.rotated = rotated
@@ -532,14 +538,14 @@ class ExplainView(QListView):
             lines.append(m18n('There is no active game'))
         else:
             i18nName = m18n(self.game.ruleset.name)
-            lines.append(m18n('Ruleset: %1', i18nName))
+            lines.append(m18n('%1', i18nName))
             lines.append('')
             for player in self.game.players:
                 pLines = []
-                if player.hand and player.hand.tileNames:
+                if player.hand and player.hand.hasTiles():
                     total = player.hand.total()
                     if total:
-                        pLines = ['%s: %s' % (player.localName, total)]
+                        pLines = ['%s - %s' % (player.localName, total)]
                         for line in player.hand.explain():
                             pLines.append('- ' + line)
                 elif player.handTotal:
@@ -721,8 +727,7 @@ class PenaltyDialog(QDialog):
 
 class ScoringDialog(QWidget):
     """a dialog for entering the scores"""
-    # pylint: disable=R0902
-    # pylint we need more than 10 instance attributes
+    # pylint: disable=too-many-instance-attributes
 
     scoringClosed = pyqtSignal()
 
@@ -875,7 +880,7 @@ class ScoringDialog(QWidget):
         """returns the currently selected last tile"""
         idx = self.cbLastTile.currentIndex()
         if idx >= 0:
-            return str(self.cbLastTile.itemData(idx).toString())
+            return Tile(self.cbLastTile.itemData(idx).toPyObject())
 
     def closeEvent(self, event):
         """the user pressed ALT-F4"""
@@ -958,7 +963,7 @@ class ScoringDialog(QWidget):
 
     def computeScores(self):
         """if tiles have been selected, compute their value"""
-        # pylint: disable=R0912
+        # pylint: disable=too-many-branches
         # too many branches
         if not self.game:
             return
@@ -995,8 +1000,8 @@ class ScoringDialog(QWidget):
                 self.game.winner = None
             self.spValues[idx].blockSignals(False)
             self.wonBoxes[idx].blockSignals(False)
-        if InternalParameters.field.explainView:
-            InternalParameters.field.explainView.refresh(self.game)
+        if Internal.field.explainView:
+            Internal.field.explainView.refresh(self.game)
 
     def __lastMeldContent(self):
         """prepare content for lastmeld combo"""
@@ -1007,10 +1012,10 @@ class ScoringDialog(QWidget):
             pairs = []
             for meld in self.game.winner.hand.melds:
                 if len(meld) < 4:
-                    pairs.extend(meld.pairs)
+                    pairs.extend(meld)
             for tile in winnerTiles:
-                if tile.element in pairs and not tile.isBonus():
-                    lastTiles.add(tile.element)
+                if tile.tile in pairs and not tile.isBonus():
+                    lastTiles.add(tile.tile)
         return lastTiles, winnerTiles
 
     def __fillLastTileComboWith(self, lastTiles, winnerTiles):
@@ -1031,11 +1036,11 @@ class ScoringDialog(QWidget):
         self.__tilePixMaps = []
         shownTiles = set()
         for tile in winnerTiles:
-            if tile.element in lastTiles and tile.element not in shownTiles:
-                shownTiles.add(tile.element)
-                self.cbLastTile.addItem(QIcon(tile.graphics.pixmapFromSvg(pmSize, withBorders=False)),
-                        '', QVariant(tile.element))
-                if indexedTile == tile.element:
+            if tile.tile in lastTiles and tile.tile not in shownTiles:
+                shownTiles.add(tile.tile)
+                self.cbLastTile.addItem(QIcon(tile.pixmapFromSvg(pmSize, withBorders=False)),
+                        '', QVariant(tile.tile))
+                if indexedTile == tile.tile:
                     restoredIdx = self.cbLastTile.count() - 1
         if not restoredIdx and indexedTile:
             # try again, maybe the tile changed between concealed and exposed
@@ -1081,10 +1086,10 @@ class ScoringDialog(QWidget):
             pixMap.fill(Qt.transparent)
             self.__meldPixMaps.append(pixMap)
             painter = QPainter(pixMap)
-            for element in meld.pairs:
+            for element in meld:
                 painter.drawPixmap(0, 0,
                     winner.handBoard.tilesByElement(element) \
-                    [0].graphics.pixmapFromSvg(QSize(faceWidth, faceHeight), withBorders=False))
+                    [0].pixmapFromSvg(QSize(faceWidth, faceHeight), withBorders=False))
                 painter.translate(QPointF(faceWidth, 0.0))
             self.cbLastMeld.addItem(QIcon(pixMap), '', QVariant(meld.joined))
             if indexedMeld == meld.joined:
@@ -1129,9 +1134,9 @@ class ScoringDialog(QWidget):
                 return
             if self.cbLastTile.count() == 0:
                 return
-            lastTile = InternalParameters.field.computeLastTile()
+            lastTile = Internal.field.computeLastTile()
             winnerMelds = [m for m in self.game.winner.hand.melds if len(m) < 4 \
-                and lastTile in m.pairs]
+                and lastTile in m]
             assert len(winnerMelds)
             if len(winnerMelds) == 1:
                 self.cbLastMeld.addItem(QIcon(), '', QVariant(winnerMelds[0].joined))
@@ -1152,6 +1157,9 @@ class ScoringDialog(QWidget):
         self.updateManualRules()
         self.computeScores()
         self.validate()
+        for player in self.game.players:
+                self.game.wall.decoratePlayer(player)
+        Internal.field.updateGUI()
 
     def validate(self):
         """update the status of the OK button"""
