@@ -40,8 +40,7 @@ PROGRAM = None
 
 class Regex(unittest.TestCase):
     """tests lots of hand examples. We might want to add comments which test should test which rule"""
-    # pylint: disable=R0904
-    # pylint - we need more than 40 public methods
+    # pylint: disable=too-many-public-methods
 
     def __init__(self, arg):
         unittest.TestCase.__init__(self, arg)
@@ -333,7 +332,7 @@ class Regex(unittest.TestCase):
                 completedHands = hand.callingHands(99)
                 testSays = ''.join(sorted(set(x.lastTile for x in completedHands))).lower()
                 if idx >= len(completingTiles):
-                    idx %= len(RULESETS) / 2
+                    idx %= len(RULESETS) // 2
                 completingTiles = completingTiles[idx]
                 self.assert_(testSays == completingTiles,
                     '%s: %s is completed by %s but test says %s' % (
@@ -395,6 +394,12 @@ class Regex(unittest.TestCase):
         """specials for chinese classical BMJA"""
         self.scoreTest(r'RS1S1S5S6S7S8S9WeWsWwWn s2s3s4 Msw Ls3s2s3s4', [Score(), Score(limits=1)])
 
+    def testLastTile(self):
+        """will the best last meld be chosen?"""
+        self.scoreTest(r'wewewe s1s1s1 b9b9b9 RC1C1C1C2C3 Mee LC1', [Score(38, 2), Score(34, 2)])
+        self.scoreTest(r'wewewe s1s1s1 b9b9b9 RC1C2C3C3C3 Mee LC3', [Score(40, 2), Score(34, 2)])
+        self.scoreTest(r'b5b6b7 s1s1s1 RB8C6C7C5B8B8C7C7 Mwew LC7', [Score(32, 0), Score(0)])
+
     def scoreTest(self, string, expected, totals=None):
         """execute one scoreTest test"""
         for idx, ruleset in enumerate(RULESETS):
@@ -409,7 +414,7 @@ class Regex(unittest.TestCase):
             if isinstance(expected, list):
                 expIdx = idx
                 if expIdx >= len(expected):
-                    expIdx %= len(RULESETS) / 2
+                    expIdx %= len(RULESETS) // 2
                 exp = expected[expIdx]
             else:
                 exp = expected
@@ -444,7 +449,7 @@ class Regex(unittest.TestCase):
 class TstProgram(unittest.TestProgram):
     """we want global access to this program so we can check for verbosity in our tests"""
     def __init__(self, *args, **kwargs):
-        global PROGRAM # pylint: disable=W0603
+        global PROGRAM # pylint: disable=global-statement
         PROGRAM = self
         unittest.TestProgram.__init__(self, *args, **kwargs)
 
